@@ -44,6 +44,7 @@ cb_resize (Totem * totem)
 	pane = GTK_WIDGET (gtk_builder_get_object (totem->xml, "tmw_main_pane"));
 	gtk_widget_style_get_property (pane, "handle-size", &gvalue_size);
 	handle_size = g_value_get_int (&gvalue_size);
+	g_value_unset (&gvalue_size);
 	
 	if (totem->sidebar_shown) {
 		w += totem->sidebar->allocation.width + handle_size;
@@ -59,6 +60,7 @@ void
 totem_sidebar_toggle (Totem *totem, gboolean state)
 {
 	GtkAction *action;
+	GtkWidget *box, *arrow;
 
 	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (totem->sidebar)) == state)
 		return;
@@ -73,10 +75,10 @@ totem_sidebar_toggle (Totem *totem, gboolean state)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), state);
 	totem_signal_unblock_by_data (G_OBJECT (action), totem);
 
-	gconf_client_set_bool (totem->gc,
-				GCONF_PREFIX"/sidebar_shown",
-				state,
-				NULL);
+	box = GTK_WIDGET (gtk_builder_get_object (totem->xml, "tmw_sidebar_button_hbox"));
+	arrow = g_object_get_data (G_OBJECT (box), "arrow");
+	gtk_arrow_set (GTK_ARROW (arrow), state ? GTK_ARROW_LEFT : GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
+
 	totem->sidebar_shown = state;
 	cb_resize(totem);
 }
@@ -93,6 +95,20 @@ totem_sidebar_is_visible (Totem *totem)
 	return totem->sidebar_shown;
 }
 
+gboolean
+totem_sidebar_is_focused (Totem *totem)
+{
+	GtkWidget *focused;
+
+	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
+	if (focused != NULL && gtk_widget_is_ancestor
+			(focused, GTK_WIDGET (totem->sidebar)) != FALSE) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 void
 totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 {
@@ -127,8 +143,15 @@ totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 	gtk_widget_show_all (totem->sidebar);
 	gtk_widget_realize (totem->sidebar);
 
-	if (!visible)
+	if (!visible) {
 		gtk_widget_hide (totem->sidebar);
+	} else {
+		GtkWidget *box, *arrow;
+
+		box = GTK_WIDGET (gtk_builder_get_object (totem->xml, "tmw_sidebar_button_hbox"));
+		arrow = g_object_get_data (G_OBJECT (box), "arrow");
+		gtk_arrow_set (GTK_ARROW (arrow), GTK_ARROW_LEFT, GTK_SHADOW_NONE);
+	}
 }
 
 char *
